/*

					Logic Scope AVR

					Support Firmware

			Copyrights 2009,  Yoshi Hiroshi

				DEVICE: ATtiny2313
				FUNCTION: Cycle Checker
					PD6,PD4,PD3,PD2Őݒ肳ꂽœ삷
					WrbgoCiJE^
						PB0:ŉʃrbg
						PB7:ŏʃrbg

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

volatile unsigned char previousSwitch;

void CheckSwitch()
{
	unsigned int newOCRA;
	unsigned char newTCCRB;
	unsigned char DIPSW;
	DIPSW = (PIND & 0b01011100) >> 2;
	if(previousSwitch != DIPSW)
	{
		TCCR1B = 0;
		switch(DIPSW)
		{
			case 0:
				newOCRA = 29;			// 1.5uS
				newTCCRB = _BV(WGM12) | _BV(CS10);
				break;
			case 1:
				newOCRA = 39;			// 2uS
				newTCCRB = _BV(WGM12) | _BV(CS10);
				break;
			case 2:
				newOCRA = 59;			// 3uS
				newTCCRB = _BV(WGM12) | _BV(CS10);
				break;
			case 3:
				newOCRA = 99;			// 5uS
				newTCCRB = _BV(WGM12) | _BV(CS10);
				break;
			case 4:
				newOCRA = 299;		// 15uS
				newTCCRB = _BV(WGM12) | _BV(CS10);
				break;
			case 5:
				newOCRA = 399;		// 20uS
				newTCCRB = _BV(WGM12) | _BV(CS10);
				break;
			case 6:
				newOCRA = 599;		// 30uS
				newTCCRB = _BV(WGM12) | _BV(CS10);
				break;
			case 7:
				newOCRA = 999;		// 50uS
				newTCCRB = _BV(WGM12) | _BV(CS10);
				break;
			case 16:	// 1-000
				newOCRA = 14;			// 15 * 10uS
				newTCCRB = _BV(WGM12) | _BV(CS12) | _BV(CS11);
				break;
			case 17:	// 1-001
				newOCRA = 19;			// 20 * 10uS
				newTCCRB = _BV(WGM12) | _BV(CS12) | _BV(CS11);
				break;
			case 18:	// 1-010
				newOCRA = 29;			// 30 * 10uS
				newTCCRB = _BV(WGM12) | _BV(CS12) | _BV(CS11);
				break;
			case 19:	// 1-011
				newOCRA = 49;			// 50 * 10uS
				newTCCRB = _BV(WGM12) | _BV(CS12) | _BV(CS11);
				break;
			case 20:	// 1-100
				newOCRA = 149;		// 150 * 10uS
				newTCCRB = _BV(WGM12) | _BV(CS12) | _BV(CS11);
				break;
			case 21:	// 1-101
				newOCRA = 199;		// 200 * 10uS
				newTCCRB = _BV(WGM12) | _BV(CS12) | _BV(CS11);
				break;
			case 22:	// 1-110
				newOCRA = 299;		// 300 * 10uS
				newTCCRB = _BV(WGM12) | _BV(CS12) | _BV(CS11);
				break;
			case 23:	// 1-111
			default:
				newOCRA = 499;		// 500 * 10uS
				newTCCRB = _BV(WGM12) | _BV(CS12) | _BV(CS11);
				break;
		}
		previousSwitch = DIPSW;
		OCR1A = newOCRA;
		TCNT1 = newOCRA;
		TCCR1B = newTCCRB;
	}
}

ISR(TIMER1_COMPA_vect, ISR_NAKED)
{
	PORTB += 1;
	reti();
}

ISR(INT0_vect)
{
	CheckSwitch();
}

ISR_ALIAS(INT1_vect, INT0_vect);


int main(void)
{
	previousSwitch = 0xFF;
	CLKPR = _BV(CLKPCE);
	CLKPR = 0;

	MCUCR = 0x05;
	GIMSK = 0xC0;

	DDRB = 0xFF;
	DDRD  = 0b00100000;
	PORTD = 0b01011100;

	OCR0A  = 99;			// 10uS
	TCCR0A = _BV(COM0B0) | _BV(WGM01);
	TCCR0B = _BV(CS00);

	//OCR1A  = 29;			// 1.5uS
	TCCR1A = 0;
	//TCCR1B = _BV(WGM12) | _BV(CS10);
	CheckSwitch();
	TIMSK = _BV(OCIE1A);
	set_sleep_mode(SLEEP_MODE_IDLE);
	while(1)
	{
		sleep_enable();
		sei();
		sleep_cpu();
		cli();
		sleep_disable();

	}
}
