/*

					Logic Scope AVR

			Copyrights 2009,  Yoshi Hiroshi


*/

/*
--------------------------------------------------
//		I/O PORT ASSIGNMENT
--------------------------------------------------
*/

//-----------------------------------------------------------------------------
//================================
#if defined (__AVR_ATmega644P__)
//================================
/*
--------------------------------------------------
//		BUFFER SIZE & HARWARE BIT MASK
--------------------------------------------------
*/
#define LOG_DATA_BUFFER_SIZE		(1024 * 3 + 512 + 256)
#define HARDWARE_BIT_MASK			(0xFF)
#define COMMAND_BUFFER_SIZE			20

// PIN ASSIGNMENT
#define LOGDATA_PIN					PINA
#define LOGDATA_DDR					DDRA
#define LOGDATA_PORT				PORTA
#define LOGDATA_PORT_INIT			{LOGDATA_DDR &= ~HARDWARE_BIT_MASK;LOGDATA_PORT = HARDWARE_BIT_MASK;}

// BAUD RATE OPTION
#define BAUD_RATE_OPTION_PORT		PORTB
#define BAUD_RATE_OPTION_PIN		PINB
#define BAUD_RATE_OPTION_DDR		DDRB
#define BAUD_RATE_OPTION_MASK		0x07
#define BAUD_RATE_OPTION_INIT		{BAUD_RATE_OPTION_DDR &= ~BAUD_RATE_OPTION_MASK;BAUD_RATE_OPTION_PORT |= BAUD_RATE_OPTION_MASK;}

// LED PIN ASSIGNMENT
#define LOGSTATUS_LED_PORT			PORTD
#define LOGSTATUS_LED_DDR			DDRD
#define LOGSTATUS_LED_WAIT			6
#define LOGSTATUS_LED_DDR_INIT		LOGSTATUS_LED_DDR |= _BV(LOGSTATUS_LED_WAIT)
#define BUSY_LED_WAIT_TRIGGER_ON	LOGSTATUS_LED_PORT &= ~_BV(LOGSTATUS_LED_WAIT)
#define BUSY_LED_WAIT_TRIGGER_OFF	LOGSTATUS_LED_PORT |= _BV(LOGSTATUS_LED_WAIT)
#define BUSY_LED_WAIT_TRIGGER_INV	LOGSTATUS_LED_PORT ^= _BV(LOGSTATUS_LED_WAIT)

#define LOG_STOP_INT_vect			PCINT3_vect
#define LOG_STOP_INT_ENABLE			PCIFR |= _BV(PCIF3);PCICR |= _BV(PCIE3);
#define LOG_STOP_INT_DISABLE		PCICR &= ~_BV(PCIF3);
#define LOG_STOP_INT_INIT			{LOG_STOP_INT_DISABLE;PCMSK3 = _BV(PCINT26);}

#endif
//================================


//-----------------------------------------------------------------------------
//================================
#if defined (__AVR_ATmega128__)
//================================
/*
--------------------------------------------------
//		BUFFER SIZE & HARWARE BIT MASK
--------------------------------------------------
*/
#define LOG_DATA_BUFFER_SIZE		(1024 * 30)
#define HARDWARE_BIT_MASK			(0xFF)
#define COMMAND_BUFFER_SIZE			20

// PIN ASSIGNMENT
#define LOGDATA_PIN					PINF
#define LOGDATA_DDR					DDRF
#define LOGDATA_PORT				PORTF
#define LOGDATA_PORT_INIT			{LOGDATA_DDR &= ~HARDWARE_BIT_MASK;LOGDATA_PORT = HARDWARE_BIT_MASK;}

// BAUD RATE OPTION
#define BAUD_RATE_OPTION_PORT		PORTB
#define BAUD_RATE_OPTION_PIN		PINB
#define BAUD_RATE_OPTION_DDR		DDRB
#define BAUD_RATE_OPTION_MASK		0x07
#define BAUD_RATE_OPTION_INIT		{BAUD_RATE_OPTION_DDR &= ~BAUD_RATE_OPTION_MASK;BAUD_RATE_OPTION_PORT |= BAUD_RATE_OPTION_MASK;}

// LED PIN ASSIGNMENT
#define LOGSTATUS_LED_PORT			PORTG
#define LOGSTATUS_LED_DDR			DDRG
#define LOGSTATUS_LED_WAIT			4
#define LOGSTATUS_LED_DDR_INIT		LOGSTATUS_LED_DDR |= _BV(LOGSTATUS_LED_WAIT)
#define BUSY_LED_WAIT_TRIGGER_ON	LOGSTATUS_LED_PORT &= ~_BV(LOGSTATUS_LED_WAIT)
#define BUSY_LED_WAIT_TRIGGER_OFF	LOGSTATUS_LED_PORT |= _BV(LOGSTATUS_LED_WAIT)
#define BUSY_LED_WAIT_TRIGGER_INV	LOGSTATUS_LED_PORT ^= _BV(LOGSTATUS_LED_WAIT)

#define LOG_STOP_INT_vect			INT2_vect
#define LOG_STOP_INT_ENABLE			EIFR |= _BV(INTF2);EIMSK |= _BV(INT2);
#define LOG_STOP_INT_DISABLE		EIMSK &= ~_BV(INT2);
#define LOG_STOP_INT_INIT			{LOG_STOP_INT_DISABLE;EICRA = _BV(ISC21);}

#endif
//================================


//-----------------------------------------------------------------------------

#define NORMAL_LOG_TCNT				TCNT1
#define NORMAL_LOG_TCCRA			TCCR1A
#define NORMAL_LOG_TCCRB			TCCR1B
#define NORMAL_LOG_OCRA				OCR1A

//================================
#if defined (__AVR_ATmega644P__)
#define NORMAL_LOG_TIMSK			TIMSK1
#endif
//================================
#if defined (__AVR_ATmega128__)
#define NORMAL_LOG_TIMSK			TIMSK
#endif
//================================

#define NORMAL_LOG_OCIEA				OCIE1A
#define NORMAL_LOG_TCNT_SET(val)	NORMAL_LOG_TCNT = (val)
#define NORMAL_LOG_OCRA_SET(val)	NORMAL_LOG_OCRA  = (val)
#define NORMAL_LOG_TIMER_START		NORMAL_LOG_TCCRA = 0;NORMAL_LOG_TCCRB = _BV(WGM12) | _BV(CS10)
#define NORMAL_LOG_ENABLE_INTERRUPT	NORMAL_LOG_TIMSK |= _BV(NORMAL_LOG_OCIEA)	
#define IS_LOG_BUSY					(lcb.busy != 0) 


/*
--------------------------------------------------
*/
typedef struct _logControl
{
	volatile unsigned char busy;
	unsigned char commandBuffer[COMMAND_BUFFER_SIZE];
	unsigned char HardwareMask;
	volatile unsigned char TriggerMask;
	volatile unsigned char TriggerPattern;
	long period;
	unsigned char logPeriod[4];
	unsigned char* buff;
	unsigned char* buffPtr;
	unsigned int  buffSize;
	volatile int  logCounter;
	volatile unsigned char dataReady;
	volatile long busyCounter;
	volatile long busyCounterMax;
} logControl;

extern logControl lcb;

